@@ -3,17 +3,13 @@
 # GitHub: https://github.com/baekalfen/PyBoy
 #
 
-import os
 import logging
+import os
 
 LOGLEVEL = os.environ.get("PYBOY_LOGLEVEL", "INFO")
 
 handler = logging.StreamHandler()
-handler.setFormatter(
-    logging.Formatter(
-        "%(relativeCreated)-8d %(name)-30s %(levelname)-8s %(message)s"
-        )
-    )
+handler.setFormatter(logging.Formatter("%(relativeCreated)-8d %(name)-30s %(levelname)-8s %(message)s"))
 
 logger = logging.getLogger("pyboy")
 logger.setLevel(LOGLEVEL)
