@@ -6,6 +6,7 @@
 The core module of the emulator
 """
 
+import logging
 import os
 import time
 
@@ -16,8 +17,8 @@ from pyboy.utils import IntIOWrapper, WindowEvent
 
 from . import botsupport
 from .core.mb import Motherboard
-from pyboy.logger import logger
 
+logger = logging.getLogger(__name__)
 
 SPF = 1 / 60. # inverse FPS (frame-per-second)
 
@@ -30,8 +31,16 @@ defaults = {
 
 class PyBoy:
     def __init__(
-            self, gamerom_file, *, bootrom_file=None, profiling=False, disable_renderer=False, sound=False,
-            randomize=False, **kwargs
+        self,
+        gamerom_file,
+        *,
+        bootrom_file=None,
+        profiling=False,
+        disable_renderer=False,
+        sound=False,
+        cgb=None,
+        randomize=False,
+        **kwargs
     ):
         """
         PyBoy is loadable as an object in Python. This means, it can be initialized from another script, and be
@@ -42,21 +51,26 @@ class PyBoy:
         interfaces. All other parts of the emulator, are subject to change.
 
         A range of methods are exposed, which should allow for complete control of the emulator. Please open an issue on
-        GitHub, if other methods are needed for your projects. Take a look at `interface_example.py` or `tetris_bot.py`
-        for a crude "bot", which interacts with the game.
+        GitHub, if other methods are needed for your projects. Take a look at the files in `examples/` for a crude
+        "bots", which interact with the game.
 
         Only the `gamerom_file` argument is required.
 
         Args:
-            gamerom_file (str): Filepath to a game-ROM for the original Game Boy.
+            gamerom_file (str): Filepath to a game-ROM for Game Boy or Game Boy Color.
 
         Kwargs:
             bootrom_file (str): Filepath to a boot-ROM to use. If unsure, specify `None`.
             profiling (bool): Profile the emulator and report opcode usage (internal use).
             disable_renderer (bool): Can be used to optimize performance, by internally disable rendering of the screen.
             color_palette (tuple): Specify the color palette to use for rendering.
+
+        Other keyword arguments may exist for plugins that are not listed here. They can be viewed with the
+        `parser_arguments()` method in the pyboy.plugins.manager module, or by running pyboy --help in the terminal.
         """
 
+        self.initialized = False
+
         for k, v in defaults.items():
             if k not in kwargs:
                 kwargs[k] = kwargs.get(k, defaults[k])
@@ -67,10 +81,11 @@ class PyBoy:
 
         self.mb = Motherboard(
             gamerom_file,
-            bootrom_file,
+            bootrom_file or kwargs.get("bootrom"), # Our current way to provide cli arguments is broken
             kwargs["color_palette"],
             disable_renderer,
             sound,
+            cgb,
             randomize=randomize,
             profiling=profiling,
         )
@@ -87,13 +102,15 @@ class PyBoy:
         self.paused = False
         self.events = []
         self.old_events = []
-        self.done = False
+        self.quitting = False
+        self.stopped = False
         self.window_title = "PyBoy"
 
         ###################
         # Plugins
 
         self.plugin_manager = PluginManager(self, self.mb, kwargs)
+        self.initialized = True
 
     def tick(self):
         """
@@ -105,38 +122,43 @@ class PyBoy:
 
         _Open an issue on GitHub if you need finer control, and we will take a look at it._
         """
+        if self.stopped:
+            return True
 
-        t_start = time.perf_counter() # Change to _ns when PyPy supports it
+        t_start = time.perf_counter_ns()
         self._handle_events(self.events)
-        t_pre = time.perf_counter()
-        self.frame_count += 1
+        t_pre = time.perf_counter_ns()
         if not self.paused:
-            self.mb.tickframe()
-        t_tick = time.perf_counter()
+            if self.mb.tick():
+                # breakpoint reached
+                self.plugin_manager.handle_breakpoint()
+            else:
+                self.frame_count += 1
+        t_tick = time.perf_counter_ns()
         self._post_tick()
-        t_post = time.perf_counter()
+        t_post = time.perf_counter_ns()
 
-        secs = t_pre - t_start
-        self.avg_pre = 0.9 * self.avg_pre + 0.1*secs
+        nsecs = t_pre - t_start
+        self.avg_pre = 0.9 * self.avg_pre + (0.1*nsecs/1_000_000_000)
 
-        secs = t_tick - t_pre
-        self.avg_tick = 0.9 * self.avg_tick + 0.1*secs
+        nsecs = t_tick - t_pre
+        self.avg_tick = 0.9 * self.avg_tick + (0.1*nsecs/1_000_000_000)
 
-        secs = t_post - t_tick
-        self.avg_post = 0.9 * self.avg_post + 0.1*secs
+        nsecs = t_post - t_tick
+        self.avg_post = 0.9 * self.avg_post + (0.1*nsecs/1_000_000_000)
 
-        return self.done
+        return self.quitting
 
     def _handle_events(self, events):
         # This feeds events into the tick-loop from the window. There might already be events in the list from the API.
         events = self.plugin_manager.handle_events(events)
         for event in events:
             if event == WindowEvent.QUIT:
-                self.done = True
+                self.quitting = True
             elif event == WindowEvent.RELEASE_SPEED_UP:
                 # Switch between unlimited and 1x real-time emulation speed
                 self.target_emulationspeed = int(bool(self.target_emulationspeed) ^ True)
-                logger.info("Speed limit: %s" % self.target_emulationspeed)
+                logger.debug("Speed limit: %s" % self.target_emulationspeed)
             elif event == WindowEvent.STATE_SAVE:
                 with open(self.gamerom_file + ".state", "wb") as f:
                     self.mb.save_state(IntIOWrapper(f))
@@ -193,7 +215,7 @@ class PyBoy:
     def _update_window_title(self):
         avg_emu = self.avg_pre + self.avg_tick + self.avg_post
         self.window_title = "CPU/frame: %0.2f%%" % ((self.avg_pre + self.avg_tick) / SPF * 100)
-        self.window_title += " Emulation: x%d" % (round(SPF / avg_emu) if avg_emu != 0 else 0)
+        self.window_title += " Emulation: x%s" % (round(SPF / avg_emu) if avg_emu > 0 else "INF")
         if self.paused:
             self.window_title += "[PAUSED]"
         self.window_title += self.plugin_manager.window_title()
@@ -202,6 +224,12 @@ class PyBoy:
     def __del__(self):
         self.stop(save=False)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.stop()
+
     def stop(self, save=True):
         """
         Gently stops the emulator and all sub-modules.
@@ -210,11 +238,13 @@ class PyBoy:
             save (bool): Specify whether to save the game upon stopping. It will always be saved in a file next to the
                 provided game-ROM.
         """
-        logger.info("###########################")
-        logger.info("# Emulator is turning off #")
-        logger.info("###########################")
-        self.plugin_manager.stop()
-        self.mb.stop(save)
+        if self.initialized and not self.stopped:
+            logger.info("###########################")
+            logger.info("# Emulator is turning off #")
+            logger.info("###########################")
+            self.plugin_manager.stop()
+            self.mb.stop(save)
+            self.stopped = True
 
     def _cpu_hitrate(self):
         return self.mb.cpu.hitrate
@@ -249,7 +279,7 @@ class PyBoy:
             action_type (str): Define how the agent will interact with button inputs
             * `"press"`: The agent will only press inputs for 1 frame an then release it.
             * `"toggle"`: The agent will toggle inputs, first time it press and second time it release.
-            * `"all"`: The agent have acces to all inputs, press and release are separated.
+            * `"all"`: The agent have access to all inputs, press and release are separated.
 
             simultaneous_actions (bool): Allow to inject multiple input at once. This dramatically increases the action_space: \\(n \\rightarrow 2^n\\)
 
@@ -413,6 +443,23 @@ class PyBoy:
 
         self.mb.load_state(IntIOWrapper(file_like_object))
 
+    def screen_image(self):
+        """
+        Shortcut for `pyboy.botsupport_manager.screen.screen_image`.
+
+        Generates a PIL Image from the screen buffer.
+
+        Convenient for screen captures, but might be a bottleneck, if you use it to train a neural network. In which
+        case, read up on the `pyboy.botsupport` features, [Pan Docs](http://bgb.bircd.org/pandocs.htm) on tiles/sprites,
+        and join our Discord channel for more help.
+
+        Returns
+        -------
+        PIL.Image:
+            RGB image of (160, 144) pixels
+        """
+        return self.botsupport_manager().screen().screen_image()
+
     def _serial(self):
         """
         Provides all data that has been sent over the serial port since last call to this function.
@@ -433,6 +480,8 @@ class PyBoy:
 
         A `target_speed` of `0` means unlimited. I.e. fastest possible execution.
 
+        Some window types do not implement a frame-limiter, and will always run at full speed.
+
         Args:
             target_speed (int): Target emulation speed as multiplier of real-time.
         """
@@ -456,4 +505,7 @@ class PyBoy:
         """
         Disable or enable rendering
         """
-        self.mb.disable_renderer = not value
+        self.mb.lcd.disable_renderer = not value
+
+    def _is_cpu_stuck(self):
+        return self.mb.cpu.is_stuck
